//
//  main.c
//  carbonUSBsample
//
//  Created by Jay Dolan on 5/12/06.
//  Copyright __MyCompanyName__ 2006. All rights reserved.
//

#include <Carbon/Carbon.h>
#include "aiousb.h"

//some handy functions for Carbon Controls
//the first two are from "Beginning Mac OS X Programming" by Micheal Trent and Drew
//McCormack. The second two were developed by Jay Dolan at ACCES
CFStringRef GetControlCFString(WindowRef window, int controlNum);
void SetTextViewCFString (WindowRef window, int controlNum, CFStringRef string);
CFStringRef myGetControlTitle(WindowRef window, int controlNum);
void mySetControlTitle(WindowRef window, int controlNum, CFStringRef string);

//main event handler
pascal OSStatus MainEventHandler(EventHandlerCallRef handler, EventRef event, void *userData);

//my command handlers
void findButtonPushedHandler (WindowRef window);

//application signature
#define kApplicationSignature	'AIOS'

//defines for control IDs
#define kFindButton			128
#define kNumDevicesEdit		129
#define kNumDIOBytesEdit	130
#define kNumCountersEdit	131

//defines for command strings
#define kFindButtonBushed	'FIND'


int main(int argc, char* argv[])
{
    IBNibRef 		nibRef;
    WindowRef 		window;
    
    OSStatus		err;

    // Create a Nib reference passing the name of the nib file (without the .nib extension)
    // CreateNibReference only searches into the application bundle.
    err = CreateNibReference(CFSTR("main"), &nibRef);
    require_noerr( err, CantGetNibRef );
    
    // Once the nib reference is created, set the menu bar. "MainMenu" is the name of the menu bar
    // object. This name is set in InterfaceBuilder when the nib is created.
    err = SetMenuBarFromNib(nibRef, CFSTR("MenuBar"));
    require_noerr( err, CantSetMenuBar );
    
    // Then create a window. "MainWindow" is the name of the window object. This name is set in 
    // InterfaceBuilder when the nib is created.
    err = CreateWindowFromNib(nibRef, CFSTR("MainWindow"), &window);
    require_noerr( err, CantCreateWindow );

    // We don't need the nib reference anymore.
    DisposeNibReference(nibRef);
    
	EventTypeSpec eventSpec = {kEventClassCommand, kEventCommandProcess};
	InstallWindowEventHandler (window,
							   NewEventHandlerUPP(MainEventHandler),
							   1,
							   &eventSpec,
							   (void*)window,
							   NULL);
	
    // The window was created hidden so show it.
    ShowWindow( window );
    
    // Call the event loop
    RunApplicationEventLoop();

CantCreateWindow:
CantSetMenuBar:
CantGetNibRef:
	return err;
}

pascal OSStatus MainEventHandler(EventHandlerCallRef handler, EventRef event, void *userData)
{
	OSStatus result = eventNotHandledErr;
	HICommand command;
	
	GetEventParameter(event,
					  kEventParamDirectObject,
					  typeHICommand,
					  NULL,
					  sizeof(HICommand),
					  NULL,
					  &command);
	
	switch (command.commandID)
	{
		case kFindButtonBushed:
			findButtonPushedHandler((WindowRef)userData);
			result = noErr;
			break;
			
	};
	
	return result;
}

CFStringRef GetControlCFString(WindowRef window, int controlNum)
{
	ControlID controlID = {kApplicationSignature, controlNum};
	ControlRef control = NULL;
	OSStatus err = noErr;
	CFStringRef string = NULL;
	
	err = GetControlByID (window, &controlID, &control);
	
	if (err == noErr)
	{
		Size actualSize;
		
		err = GetControlData(control,
							 kControlEntireControl,
							 kControlEditTextCFStringTag,
							 sizeof(string),
							 &string,
							 &actualSize);
		
		
		if (err != noErr)
			string = NULL;
		
	}
	
	return string;
}

void SetTextViewCFString (WindowRef window, int controlNum, CFStringRef string)
{
	HIViewID viewID = {kApplicationSignature, controlNum };
	HIViewRef textView;
	OSStatus err = noErr;
	
	err = HIViewFindByID (HIViewGetRoot (window), viewID, &textView);
	
	if (err == noErr)
	{
		CFIndex length = CFStringGetLength(string);
		UniChar *buffer = (UniChar *) malloc (length * sizeof(UniChar));
		
		if (buffer)
		{
			CFRange range = {0, length};
			CFStringGetCharacters (string, range, buffer);
			err = TXNSetData(HITextViewGetTXNObject(textView),
							 kTXNUnicodeTextData,
							 buffer,
							 length * sizeof(UniChar),
							 kTXNStartOffset,
							 kTXNEndOffset);
			
			free(buffer);
		}
	}
}

CFStringRef myGetControlTitle(WindowRef window, int controlNum)
{
	ControlID controlID = {kApplicationSignature, controlNum};
	ControlRef control = NULL;
	OSStatus err = noErr;
	CFStringRef string = NULL;
	
	err = GetControlByID (window, &controlID, &control);
	
	if (err == noErr)
	{
		CopyControlTitleAsCFString(control , &string);
	}
	
	return string;
}

void mySetControlTitle(WindowRef window, int controlNum, CFStringRef string)
{
	ControlID controlID = {kApplicationSignature, controlNum};
	ControlRef control = NULL;
	OSStatus err = noErr;
	
	err = GetControlByID (window, &controlID, &control);
	
	if (err == noErr)
	{
		SetControlTitleWithCFString(control, string);
	}
}

void findButtonPushedHandler (WindowRef window)
{
	int numDevices;
	UInt8 DIOBytes, counters;
	CFStringRef string;
	
	numDevices = queryDevices();
	
	string = CFStringCreateWithFormat(kCFAllocatorDefault, NULL,
									  CFSTR("%d"), numDevices);
	
	SetTextViewCFString(window, kNumDevicesEdit, string);
	
	if (numDevices != 0)
	{
		queryDeviceInfo(0, NULL, 0, NULL, &DIOBytes, &counters);
		
		string = CFStringCreateWithFormat(kCFAllocatorDefault,
										  NULL ,CFSTR("%d"), DIOBytes);
		SetTextViewCFString(window , kNumDIOBytesEdit,	string);
		
		string = CFStringCreateWithFormat(kCFAllocatorDefault, 0, 
										  CFSTR("%d"), counters);
		
		SetTextViewCFString(window, kNumCountersEdit, string);
	}
	
}
